#ifndef _MDTKEA_H /* -*- mode: C; -*- */
#define _MDTKEA_H

/*
   Copyright (c) 1997-2002 MathEngine PLC

   $Name:  $

   Date: $Date: 2003/05/05 00:57:07 $ - Revision: $Revision: 1.1 $

   This software and its accompanying manuals have been developed
   by MathEngine PLC ("MathEngine") and the copyright and all other
   intellectual property rights in them belong to MathEngine. All
   rights conferred by law (including rights under international
   copyright conventions) are reserved to MathEngine. This software
   may also incorporate information which is confidential to
   MathEngine.

   Save to the extent permitted by law, or as otherwise expressly
   permitted by MathEngine, this software and the manuals must not
   be copied (in whole or in part), re-arranged, altered or adapted
   in any way without the prior written consent of the Company. In
   addition, the information contained in the software may not be
   disseminated without the prior written consent of MathEngine.

 */

/** @file
 *
 * Kea solver header file.
 *
 * For explanation of some notation see "Programming From
 * Specifications" by Carroll Morgan.
 */

#include <MePrecision.h>
#include <MdtSolverUtils.h>

#define MdtKeaBODYVER1           100

/**
 * MdtKeaMAXBODYCONSTRAINT
 *
 * The maximum number of bodies that can be constrained by each constraint
 */
#define MdtKeaMAXBODYCONSTRAINT 2

/**
 * MdtKeaBody Flag enum.
 *
 * Any bitwise combination of these options is valid for use in
 * 'MdtKeaBody.flags'.
 */
typedef enum
{
    /**
     * Use the 'fast spin' integrator for this body.
     */
    MdtKeaBodyFlagUseFastSpin = 0x01,

    /**
     * Indicates that this body has at least one constraint affecting it.
     */
    MdtKeaBodyFlagIsConstrained = 0x02,

    /**
     * indicates that this flag should use the full MoI tensor
     * in the mass matrix rather than a spherical approximation
     */
    MdtKeaBodyFlagIsNonSpherical = 0x04,

    /**
     * indicates that this flag should use add Coriolis Forces 
     * to the body
     */
    MdtKeaBodyFlagAddCoriolisForce = 0x08
}
MdtKeaBodyFlag;
/** 
 * Force,torque pair
 */
typedef struct
{
    MeReal force[3];
    int    pad0;

    MeReal torque[3];
    int    pad1;
} MdtKeaForce;

/** 
 * Velocity,angular velocity pair
 */
typedef struct
{
    MeReal velocity[3];
    int    pad0;

    MeReal angVelocity[3];
    int    pad1;
} MdtKeaVelocity;


/**
 * Pair of force,torque pairs.
 */
typedef struct 
{
    MdtKeaForce primary_body;
    MdtKeaForce secondary_body;
} MdtKeaForcePair;

/**
 * 4*6 Jacobian matrix block,
 * in transpose wallpaper format.
 */
typedef struct 
{
    MeVector4 col[6];
} MdtKeaJBlock;

/**
 * Pair of 4*6 Jacobian matrix blocks,
 * in transpose wallpaper format.
 */
typedef MdtKeaJBlock MdtKeaJBlockPair[2];

typedef struct
{
    MdtKeaJBlockPair * elt;
    int              numElts;
    int              maxElts;
} MdtKeaJBlockPairArray;

/**
 * Row of the bl2body table
 */
typedef int MdtKeaBl2BodyRow[8]; 

typedef struct
{
    MdtKeaBl2BodyRow * elt;
    int                numElts;
    int                maxElts;
} MdtKeaBl2BodyRowArray;

/**
 * Debug data logging
 */

typedef struct
{
    /** Set writeKeaInterData in order to write keas input to a file */
    MeBool writeKeaInputData;

    /** Name of the file to dump kea's input to */
    char * writeKeaInputDataFilename;

    /** Set readKeaInterData in order to read kea's input from a file, ignoring
        the input supplied as parameters to MdtKeaCalcConstraintForces */
    MeBool readKeaInputData;

    /** Name of the file to read kea's input from */
    char * readKeaInputDataFilename;

    /** Set writeKeaInterData in order to dump intermediate results of the kea 
        calculation */
    MeBool writeKeaInterData;

    /** Name of the file to dump the intermediate results to */
    char * writeKeaInterDataFilename;

    /** Set writeKeaOutputData in order to dump the output of the kea */ 
    MeBool writeKeaOutputData;

    /** Name of the file to dump kea's to */
    char * writeKeaOutputDataFilename;

    /** frame should be set to the number of the current graphics frame */
    int    frame;

    /** The number of the frame being investigated */
    int    badFrame;

    /** The partition that is being investigated */
    int    badPartition;
    
} MdtKeaDebugDataRequest;

/**
 * Kea Parameters structure.
 */

/**
 * Sparse representation of the inverse of a 6*6 mass matrix.
 *
 * Can't use MeVector3 in this definition because we want to
 * explicitly pad so we can put invmass in the padding.
 *
 * It is absolutely vital that the sizeof this structure is an integer
 * multiple of the sizeof an MeReal, since the optimised solvers depend on
 * this.
 */
typedef struct
{
    /** Inverse of 3x3 inertia tensor - row 0. */
    MeReal invI0[3];
    /** Inverse of mass                        */
    MeReal invmass;

    /** Inverse of 3x3 inertia tensor - row 1. */
    MeReal invI1[3];
    MeReal pad1;

    /** Inverse of 3x3 inertia tensor - row 2. */
    MeReal invI2[3];
    MeReal pad2;
} MdtKeaInverseMassMatrix;

typedef struct
{
    int pad;
} MdtKeaInterData;

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Utility function for detecting the CPU Resources
 *
 */
MEPUBLIC
MeCPUResources MEAPI MdtKeaQueryCPUResources(void);

/**
 * Utility function for calculating the size of the workspace needed by
 * Kea.
 *
 * This memory is passed into Kea in the MdtKeaParameters structure.
 */
MEPUBLIC
int MEAPI MdtKeaMemoryRequired(const MeI32Array  numRowsPartition, /* Input */
                               int               max_rows,         /* Input */
                               int               max_bodies);      /* Input */

/**
 * Kea solver function.
 *
 * Takes an array of bodies and a MdtConstraints struct (which
 * defines the constraints as defined above), as well as an array
 * indicating which constraints are to be solved together as one
 * partition. It solves each partition and evolves the system.
 *
 * @param constraints A MdtConstraints struct containing constraint
 * rows generated by MdtBcl functions.
 * @param blist Array of MdtKeaBody structs to be simulated.
 * @param tlist Array of MdtKeaTransformation structs that correspond to
 * blist.
 * @param num_bodies Number of bodies/transformations in blist/tlist
 * @param parameters A MdtKeaParameters structure containing stepsize,
 * memory pool etc.
 */


MEPUBLIC
void MEAPI MdtKeaCalcConstraintForces(MdtSolverOutputData        outputData,
                                      MdtKeaInterData            interData,
                                      const MdtSolverInputData   inputData);
/*
 * Platform dependent functions for flushing and invalidating dcache lines
 */

MEPUBLIC
void MdtFlushCache(int);
MEPUBLIC
void MdtSyncDCache(void *, void *);
MEPUBLIC
void MdtInvalidDCache(void *, void *);

#ifdef __cplusplus
}
#endif

#endif
